package com.example.datadump.service;

import com.example.datadump.util.DateTimeUtil;
import com.example.datadump.util.FileUtils;
import com.taosdata.jdbc.TSDBDriver;
import org.springframework.stereotype.Service;

import java.io.File;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.*;

@Service
public class DumpTDEngine {
    private static List<Map<String, String>> convertList(ResultSet rs) throws Exception {
        List<Map<String, String>> list = new ArrayList<>();
        ResultSetMetaData md = rs.getMetaData();//获取键名
        int columnCount = md.getColumnCount();//获取行的数量
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSXXX");
        while (rs.next()) {
            Map<String, String> rowData = new LinkedHashMap<>();//声明Map
            for (int i = 1; i <= columnCount; i++) {
                if ("ts".equals(md.getColumnName(i))) {
                    //把2021-01-01 00:10:00.0 转换成 2021-01-01T08:00:00.000+08:00'
                    Date date = format.parse(rs.getString(i));
                    rowData.put(md.getColumnName(i), format1.format(date));
                } else {
                    rowData.put(md.getColumnName(i), rs.getString(i) == null ? "NULL" : rs.getString(i));//获取键名及值
                }
            }
            list.add(rowData);
        }
        return list;
    }

    /**
     * 获取列头
     *
     * @param a
     * @return
     */
    private static List<String[]> getList(List<Map<String, String>> a) {
        //表头数据
        List<String[]> newResult = new ArrayList<>();
        if (!a.isEmpty()) {
            for (Map<String, String> map : a) {
                String[] colName = new String[a.get(0).size()];
                int i = 0;
                for (Map.Entry<String, String> entry : map.entrySet()) {
                    colName[i] = entry.getValue();
                    i++;
                }
                newResult.add(colName);
            }

        }
        return newResult;
    }


    /**
     * TDEngine数据库备份
     *
     * @param ip       IP地址
     * @param port     端口号
     * @param db       数据库
     * @param userName 用户名
     * @param passWord 密码
     * @param table    表名,如果多个表以逗号分隔
     * @param path     导出路径
     * @throws Exception
     */
    public void dbBackUpTDEngine(String ip, String port, String db, String userName, String passWord, String table, String path) throws Exception {
        String jdbcUrl = "jdbc:TAOS://" + ip + ":" + port + "/" + db + "?user=" + userName + "&password=" + passWord;
        Properties connProps = new Properties();
        connProps.setProperty(TSDBDriver.PROPERTY_KEY_CHARSET, "UTF-8");
        connProps.setProperty(TSDBDriver.PROPERTY_KEY_LOCALE, "en_US.UTF-8");
        connProps.setProperty(TSDBDriver.PROPERTY_KEY_TIME_ZONE, "UTC-8");
        Connection conn = DriverManager.getConnection(jdbcUrl, connProps);
        //--------------------------------------------------
        Statement ps;
        ps = conn.createStatement();
        //所有数据
        ResultSet rs = null;
        //转换过后的数据
        List<Map<String, String>> a;
        /*Timestamp timestamp = new Timestamp(DateTimeUtil.getDayStartTime(1586880000000L).getTime());
        Timestamp ytimestamp = new Timestamp(DateTimeUtil.getDayLastTime(1586966400000L - 1000).getTime());
        String start = timestamp.toString();
        String end = ytimestamp.toString();*/
        String start = DateTimeUtil.yesterdayStart();
        String end = DateTimeUtil.yesterdayEnd();
        //---------创建文件----------
        String[] csvNames = table.split(",");
        //根据表名循环
        for (String tableName : csvNames) {
            String filePath = path + File.separator + tableName;
            //如果目录下没有这个表的文件,那么就是第一次执行,只有第一次执行的时候才会导出所有的数据
            if (FileUtils.searchFile(new File(path), tableName).length == 0) {
                String sql = "select ts from " + tableName + " limit 0,1";
                rs = ps.executeQuery(sql);
                String startTime = "";
                while (rs.next()) {
                    startTime = rs.getString("ts");
                }
                String endTime = DateTimeUtil.yesterdayEnd();
                SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");//首先定义待转换的时间格式
                long startDate = format.parse(startTime).getTime();//将带转换的时间字符串转换为date类型，然后使用getTime即可获取对应的时间戳
                long endDate = format.parse(endTime).getTime();//将带转换的时间字符串转换为date类型，然后使用getTime即可获取对应的时间戳
                for (long time = startDate; time <= endDate; time += 60 * 60 * 24 * 1000) {
                    Timestamp timestamp1 = new Timestamp(DateTimeUtil.getDayStartTime(time).getTime());
                    Timestamp timestamp2 = new Timestamp(DateTimeUtil.getDayLastTime(time + 60 * 60 * 24 * 1000 - 1000).getTime());
                    String start1 = timestamp1.toString();
                    String end1 = timestamp2.toString();
                    sql = "select * from " + tableName + " where ts between '" + start1 + "' and '" + end1 + "'";
                    rs = ps.executeQuery(sql);
                    a = convertList(rs);
                    if (!a.isEmpty()) {
                        List<String[]> newResult = getList(a);
                        FileUtils.createCsvFile(newResult, filePath + " " + start1 + ".csv");
                    }
                }
            } else {
                String sql = "select * from " + tableName + " where ts between '" + start + "' and '" + end + "'";
                rs = ps.executeQuery(sql);
                //转换过后的数据
                a = convertList(rs);
                //表头数据
                if (!a.isEmpty()) {
                    List<String[]> newResult = getList(a);
                    FileUtils.createCsvFile(newResult, filePath + " " + start + ".csv");
                }
            }
        }
        if (rs != null)
            rs.close();
        if (ps != null)
            ps.close();
        conn.close();
    }


}
